/*
*********************************************************************************************************
*                                              uC/Modbus
*                                       The Embedded Modbus Stack
*
*                          (c) Copyright 2003-2009; Micrium, Inc.; Weston, FL               
*
*               All rights reserved.  Protected by international copyright laws.
*               Knowledge of the source code may NOT be used to develop a similar product.
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                        MODBUS NON-OS LAYER INTERFACE
* 
* Filename      : mb_os.c
* Version       : V2.12
* Programmer(s) : FT
*********************************************************************************************************
* Note(s)       :
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#define   MB_OS_MODULE
#include  <mb_os.h>

/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                           LOCAL DEFINES
*********************************************************************************************************
*/

#define  MB_OS_ERR_NONE          0x00
#define  MB_OS_ERR_Q_FULL        0x01
#define  MB_OS_ERR_Q_EMPTY       0x03
#define  MB_OS_ERR_TIMEOUT       0x02

/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/

#if (MODBUS_CFG_SLAVE_EN == DEF_ENABLED)
typedef struct mb_os_q {
    MODBUS_CH   **OS_Q_StoPtr;                               /* Queue storage pointer                                   */
    CPU_INT08U    OS_Q_Entries;                              /* Queue current number of entries                         */
    CPU_INT08U    OS_Q_MaxEntries;                           /* Queue max number of entries                             */
    CPU_INT08U    OS_Q_FirstEntry;                           /* Queue first entry index                                 */
    CPU_INT08U    OS_Q_LastEntry;                            /* Queue last  entry index                                 */
} MB_OS_Q;
#endif

/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/

#if (MODBUS_CFG_SLAVE_EN == DEF_ENABLED)
static  MB_OS_Q     MB_OS_RxQ;
static  MODBUS_CH  *MB_OS_RxQ_Tbl[MODBUS_CFG_MAX_CH];
#endif

/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/

#if (MODBUS_CFG_SLAVE_EN == DEF_ENABLED)
                                                                 /* Queue functions                                    */
static  void        MB_OS_Q_Create    (MB_OS_Q     *p_q,
                                       MODBUS_CH  **p_sto, 
                                       CPU_INT08U   max_entries);

static  MODBUS_CH  *MB_OS_Q_Pend      (MB_OS_Q     *p_q);

static  CPU_INT08U  MB_OS_Q_Post      (MB_OS_Q     *p_q,
                                       MODBUS_CH   *p_msg);

static  void        MB_OS_InitSlave    (void);
        void        MB_OS_RxTask       (void);

#endif

        
/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/

/*$PAGE*/
/*
*********************************************************************************************************
*                                              MB_OS_Init()
*
* Description : This function initializes the NON OS interface.  This function creates the following:
*
*               (1) A queue structure that contains the pointer to the channel(s) wher a message needs to be processed. (Modbus Slave)
*
* Argument(s) : none
*
* Return(s)   : none.
*
* Caller(s)   : MB_Init()
*
* Note(s)     : none.        
*********************************************************************************************************
*/

void  MB_OS_Init (void)
{
#if (MODBUS_CFG_SLAVE_EN  == DEF_ENABLED)
    MB_OS_InitSlave();
#endif
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                          MB_OS_InitSlave()
*
* Description : This function create a queue structure for income Rx messages. (Modbus Slave)
*
* Argument(s) : none
*
* Return(s)   : none.
*
* Caller(s)   : MB_OS_Init()
*
* Note(s)     : none.        
*********************************************************************************************************
*/

#if (MODBUS_CFG_SLAVE_EN == DEF_ENABLED)
static  void  MB_OS_InitSlave (void)
{    
    MB_OS_Q_Create(&MB_OS_RxQ, &MB_OS_RxQ_Tbl[0], MODBUS_CFG_MAX_CH);
}
#endif

/*
*********************************************************************************************************
*                                             MB_OS_Exit()
*
* Description : This function is called to terminate the RTOS interface for Modbus channels.  
*
*               No action is performed for the Non OS layer.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : MB_Exit().
*********************************************************************************************************
*/

void  MB_OS_Exit (void)
{

}


/*$PAGE*/
/*
*********************************************************************************************************
*                                              MB_OS_RxSignal()
*
* Description : This function signals the reception of a packet either from the Rx ISR(s) or the RTU timeout
*               timer(s) to indicate that a received packet needs to be processed.
*
* Argument(s) : pch     specifies the Modbus channel data structure in which a packet was received.
*
* Return(s)   : none.
*
* Caller(s)   : MB_ASCII_RxByte()
*               MB_RTU_TmrUpdate()
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  MB_OS_RxSignal (MODBUS_CH *pch)
{
    if (pch != (MODBUS_CH *)0) {        
#if (MODBUS_CFG_SLAVE_EN == DEF_ENABLED)
        MB_OS_Q_Post(&MB_OS_RxQ, 
                     pch);
#endif        
    }
}

/*$PAGE*/
/*
*********************************************************************************************************
*                                            MB_OS_RxTask()
*
* Description : This functions needs to be called frequently by the application to poll if a Rx has been received.
*              
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) If (RxTimeout == 0) MB_OS_RxTask() will check if there is an Rx message in the queue,
                    if there is not this functions will return immediately.
*********************************************************************************************************
*/

#if (MODBUS_CFG_SLAVE_EN == DEF_ENABLED)
void  MB_OS_RxTask (void)
{
    MODBUS_CH   *p_ch;       
          
    
    p_ch = (MODBUS_CH *)MB_OS_Q_Pend(&MB_OS_RxQ);                /* Wait for a Rx packet                               */
    
    if (p_ch == (MODBUS_CH *)0) {
        return;
    }    
    
    MB_RxTask(p_ch);                                            /* Process the packet received                        */                                    
    
}
#endif

/*$PAGE*/
/*
*********************************************************************************************************
*                                            MB_OS_SemCreate()
*
* Description : Creates a semaphore.
*
* Argument(s) : p_sem       it is pointer the the MB_OS_SEM structure.
*               cnt         The initial value of the semaphore
*
* Return(s)   : none.
*                            
* Caller(s)   : MB_OS_InitMaster()
*
* Return(s)   : none.
*********************************************************************************************************
*/

#if (MODBUS_CFG_MASTER_EN == DEF_ENABLED)
static  void  MB_OS_SemCreate  (MB_OS_SEM  *p_sem, 
                                CPU_INT32U  cnt)
{
    p_sem->OS_SemCnt = cnt;    
}
#endif


/*$PAGE*/
/*
*********************************************************************************************************
*                                            MB_OS_SemPost()
*
* Description : This function signals a semaphore
*
* Argument(s) : p_sem       it is pointer the the MB_OS_SEM structure.
*
* Return(s)   : none.
*                            
* Caller(s)   : MB_OS_InitMaster()
*
* Return(s)   : none.
*********************************************************************************************************
*/

#if (MODBUS_CFG_MASTER_EN == DEF_ENABLED)
static  void  MB_OS_SemPost    (MB_OS_SEM  *p_sem)
{
#if (CPU_CFG_CRITICAL_METHOD == CPU_CRITICAL_METHOD_STATUS_LOCAL)
    CPU_SR       cpu_sr = 0;
#endif    

    
    CPU_CRITICAL_ENTER();                                       /* Increment the semaphore counts value               */
    p_sem->OS_SemCnt++;
    CPU_CRITICAL_EXIT();
}
#endif


/*$PAGE*/
/*
*********************************************************************************************************
*                                            MB_OS_Q_Create()
*
* Description : this function creates a message queue.
*
* Argument(s) : p_sem       it is pointer the the MB_OS_SEM structure.
*               cnt         The initial value of the semaphore
*
* Return(s)   : none.
*                            
* Caller(s)   : MB_OS_InitMaster()
*
* Return(s)   : none.
*********************************************************************************************************
*/

#if (MODBUS_CFG_SLAVE_EN == DEF_ENABLED)
static void  MB_OS_Q_Create (MB_OS_Q     *p_q, 
                             MODBUS_CH  **p_sto, 
                             CPU_INT08U   max_entries)
{
    p_q->OS_Q_StoPtr     = p_sto;
    p_q->OS_Q_Entries    = 0;
    p_q->OS_Q_MaxEntries = max_entries;
    p_q->OS_Q_FirstEntry = 0;
    p_q->OS_Q_LastEntry  = 0;
}
#endif

/*$PAGE*/
/*
*********************************************************************************************************
*                                            MB_OS_Q_Create()
*
* Description : this function creates a message queue.
*
* Argument(s) : p_sem       it is pointer the the MB_OS_SEM structure.
*               cnt         The initial value of the semaphore
*
* Return(s)   : none.
*                            
* Caller(s)   : MB_OS_InitMaster()
*
* Return(s)   : none.
*********************************************************************************************************
*/

#if (MODBUS_CFG_SLAVE_EN == DEF_ENABLED)
static  CPU_INT08U  MB_OS_Q_Post (MB_OS_Q    *p_q,
                                  MODBUS_CH  *p_msg)
{
#if (CPU_CFG_CRITICAL_METHOD == CPU_CRITICAL_METHOD_STATUS_LOCAL)
    CPU_SR       cpu_sr = 0;
#endif   
    MODBUS_CH  **p_sto;
    CPU_INT08U   err;
 
    
    p_sto = p_q->OS_Q_StoPtr;
    err   = MB_OS_ERR_NONE;
        
    CPU_CRITICAL_ENTER();    
    if ((p_q->OS_Q_MaxEntries) > (p_q->OS_Q_Entries)) {        
        p_q->OS_Q_Entries++;

        if (p_q->OS_Q_LastEntry == (p_q->OS_Q_MaxEntries - 1)) {
            p_q->OS_Q_LastEntry = 0;
        } else {
            p_q->OS_Q_LastEntry++;
        }

        p_sto[p_q->OS_Q_LastEntry] = p_msg;       

    } else {
        err =  MB_OS_ERR_Q_FULL;   
    }
    CPU_CRITICAL_EXIT();    

    return err;
}
#endif

/*$PAGE*/
/*
*********************************************************************************************************
*                                            MB_OS_Q_Pend()
*
* Description : This function wait until at least on message receive in the queue or a timeout ocurrs.
*
* Argument(s) : p_q         Pointer to the message queue.
*
*               timeout     is an optional timeout period (in clock ticks).  If non-zero, this function will
*                           wait for the resource up to the amount of time specified by this argument.
*                           If you specify 0, however, your This funtion wait forever until the resource 
*                           becomes available (or the event occurs).
*
*               p_err       Pointer to the return error value
*
*                           MB_OS_ERR_NONE     If no error ocurred
*                           MB_OS_ERR_TIMEOUT  If a timeout ocurred
*
* Return(s)   : A pointer to the message received.
*                            
* Caller(s)   : MB_OS_RxTask()
*
* Return(s)   : none.
*********************************************************************************************************
*/

#if (MODBUS_CFG_SLAVE_EN == DEF_ENABLED)
static  MODBUS_CH  *MB_OS_Q_Pend  (MB_OS_Q  *p_q)
{    
#if (CPU_CFG_CRITICAL_METHOD == CPU_CRITICAL_METHOD_STATUS_LOCAL)
    CPU_SR       cpu_sr = 0;
#endif   

    MODBUS_CH   **p_sto;
    MODBUS_CH    *p_msg;     
    
    
    p_sto   = p_q->OS_Q_StoPtr;
    p_msg   = (MODBUS_CH *)0;

    if (p_q->OS_Q_Entries > 0) {
         CPU_CRITICAL_ENTER();

         p_msg = (p_sto[p_q->OS_Q_FirstEntry]);            

         if (p_q->OS_Q_FirstEntry == (p_q->OS_Q_MaxEntries - 1)) {
             p_q->OS_Q_FirstEntry = 0;
         } else {
             p_q->OS_Q_FirstEntry++;
         }            
            
         p_q->OS_Q_Entries--;

         CPU_CRITICAL_EXIT();   
 
    }    
    
    return (p_msg);
}
#endif

