/*
************************************************************************************************************************
*                                                      uC/OS-II
*                                                The Real-Time Kernel
*
*
*                                      (c) Copyright 2012, Micrium, Weston, FL
*                                                 All Rights Reserved
*
*                                            ARM CORTEX-M3 OS CHIP SUPPORT
*                                                    ACTEL A2Fx00 
*                                            
*
* File      : OS_CPU.H
* Version   : V3.01.1
* By        : JJL
*             FT
*
* For       : ARMv7M Cortex-M3
* Mode      : Thumb2
************************************************************************************************************************
*/

#define  OS_CSP_MODULE
#include <ucos_ii.h>
#include <csp.h>


/*$PAGE*/
/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/

#define  OS_CSP_REG_SYSTICK_CR                  (* (CPU_REG32 *)(0xE0042038))

#define  OS_CSP_MSK_SYSTICK_CR_STCLK_DIV        DEF_BIT_FIELD(2u, 28u)
#define  OS_CSP_BIT_SYSTICK_CR_STCLK_DIV_04     DEF_BIT_NONE
#define  OS_CSP_BIT_SYSTICK_CR_STCLK_DIV_08     DEF_BIT_MSK(1u, 28u)
#define  OS_CSP_BIT_SYSTICK_CR_STCLK_DIV_16     DEF_BIT_MSK(2u, 28u)
#define  OS_CSP_BIT_SYSTICK_CR_STCLK_DIV_32     DEF_BIT_MSK(3u, 28u)
#define  OS_CSP_BIT_SYSTICK_CR_NOREF            DEF_BIT_25
#define  OS_CSP_BIT_SYSTICK_CR_SKEW             DEF_BIT_24


/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/



/*$PAGE*/
/*
************************************************************************************************************************
*                                                INITIALIZE TICKER INTERRUPT
*
* Description: Initialize the the Tick Interrupt.
*
* Arguments  : none.
*
* Note(s)    : (1) This function MUST be called after OSStart() & after processor initialization.
*
*              (2) The A2Fx00 uses a SYSTIC_CR in conjunction with the systick control register within in the NVIC 
*                  module to control the behavior of the systick timer.
*
*                 The SysTick timer uses FCLK if 'NOREF' bit is set to 1, and uses STCLK if NOREF is set to 0.
************************************************************************************************************************
*/

void  OS_CSP_TickInit (void)
{
    CPU_INT32U  cnts;
    CPU_INT32U  systick_clk;    

    
    systick_clk = CSP_PM_CPU_ClkFreqGet();                      /* Get the FCLK frequency.                              */
    DEF_BIT_SET(OS_CSP_REG_SYSTICK_CR, OS_CSP_BIT_SYSTICK_CR_NOREF | OS_CSP_BIT_SYSTICK_CR_SKEW);

    cnts = (systick_clk / OS_TICKS_PER_SEC);                    /* Calculate the number of SysTick counts               */

    OS_CPU_SysTickInit(cnts);                                   /* Call the Generic OS Systick initialization           */
}


/*$PAGE*/
/*
************************************************************************************************************************
*                                               INTERRUPT CONTROLLER HANDLER
*
* Description: Handles all Cortex M3 NVIC's external sources interrupts
*
* Arguments  : src_nbr  External interrupt source number
*
* Note(s)    : none.
************************************************************************************************************************
*/

void  CSP_IntHandler (void)
{
    CPU_VOID      *p_arg;
    CSP_INT_VECT  *p_int_vect;
    CSP_DEV_NBR    src_nbr;        
    CPU_INT32U     reg_val;
    CPU_FNCT_PTR   isr_fnct;    
#if (CSP_CFG_INT_ISR_EXEC_MEAS_EN == DEF_ENABLED) && \
    (CPU_CFG_TS_EN                == DEF_ENABLED)
    CPU_TS32       ts;
#endif       
    CPU_SR_ALLOC();


    CPU_CRITICAL_ENTER();                                       /* Tell the OS that we are starting an ISR              */

    OSIntEnter();

    CPU_CRITICAL_EXIT();

    reg_val  = (CPU_REG_NVIC_ICSR & CPU_MSK_NVIC_ICSR_VECT_ACTIVE);        
    src_nbr  = (CSP_DEV_NBR)reg_val -  CPU_INT_EXT0;
    
    if (src_nbr < CSP_NBR_MAX_INT_SRC) {
        p_int_vect = &CSP_MainVectTbl[src_nbr];        
        isr_fnct   =  p_int_vect->FnctPtr;
        p_arg      =  p_int_vect->ArgPtr;   
      
        if (isr_fnct != (CPU_FNCT_PTR)0) {

#if (CSP_CFG_INT_ISR_EXEC_MEAS_EN == DEF_ENABLED) && \
    (CPU_CFG_TS_EN                == DEF_ENABLED)
        ts = CPU_TS_Get32();                                    /* Get current time stamp                               */                 

        (*isr_fnct)(p_arg);                                     /* Call the Interrupt handler                           */

        ts = CPU_TS_Get32() - ts;                               /* Compute delta time between start and end             */
                                                                /* Detect peak value                                    */
        if (p_int_vect->TimeMax < ts) {
            p_int_vect->TimeMax = ts;
        }
        CPU_CRITICAL_ENTER();
        
        p_int_vect->TimeCur = ts;                 
        
        CPU_CRITICAL_EXIT();
        
#else
        (*isr_fnct)(p_arg);     
#endif        
        }
    }

    OSIntExit();                                                /* Tell the OS that we are leaving the ISR              */
}
